@@ -12,6 +12,9 @@
 
 #include <libxml/xmlversion.h>
 #include <libxml/hash.h>
+#include <libxml/xmlstring.h>
+
+#ifdef LIBXML_SCHEMAS_ENABLED
 
 #ifdef __cplusplus
 extern "C" {
@@ -81,6 +84,16 @@ typedef enum {
     XML_RELAXNG_ERR_TEXTWRONG
 } xmlRelaxNGValidErr;
 
+/*
+ * xmlRelaxNGParserFlags:
+ *
+ * List of possible Relax NG Parser flags
+ */
+typedef enum {
+    XML_RELAXNGP_NONE = 0,
+    XML_RELAXNGP_FREE_DOC = 1,
+    XML_RELAXNGP_CRNG = 2
+} xmlRelaxNGParserFlag;
 /*
  * Interfaces for parsing.
  */
@@ -92,6 +105,10 @@ XMLPUBFUN xmlRelaxNGParserCtxtPtr XMLCALL
 XMLPUBFUN xmlRelaxNGParserCtxtPtr XMLCALL   
 		    xmlRelaxNGNewDocParserCtxt	(xmlDocPtr doc);
 
+XMLPUBFUN int XMLCALL
+		    xmlRelaxParserSetFlag	(xmlRelaxNGParserCtxtPtr ctxt,
+		    				 int flag);
+
 XMLPUBFUN void XMLCALL		
 		    xmlRelaxNGFreeParserCtxt	(xmlRelaxNGParserCtxtPtr ctxt);
 XMLPUBFUN void XMLCALL			
@@ -161,4 +178,7 @@ XMLPUBFUN int XMLCALL
 #ifdef __cplusplus
 }
 #endif
+
+#endif /* LIBXML_SCHEMAS_ENABLED */
+
 #endif /* __XML_RELAX_NG__ */
