@@ -209,7 +209,7 @@ xmlStrncmp(const xmlChar *str1, const xmlChar *str2, int len) {
     if (str1 == NULL) return(-1);
     if (str2 == NULL) return(1);
 #ifdef __GNUC__
-    tmp = strncmp(str1, str2, len);
+    tmp = strncmp((const char *)str1, (const char *)str2, len);
     return tmp;
 #else
     do {
